from typing import Optional
from uuid import UUID

from sqlalchemy import JSON, Column
from sqlmodel import Field, Relationship

from ..general.models import BaseModel


class CandidatesModel(BaseModel):
    pass


class CandidatesOrganizationModel(CandidatesModel):
    organization: UUID = Field(index=True)


class Document(CandidatesOrganizationModel, table=True):
    type: str = Field(index=True)
    blob_name: str = Field(index=True)

    candidate_id: Optional[UUID] = Field(foreign_key="candidate.id", ondelete="SET NULL", index=True, nullable=True)
    candidate: "Candidate" = Relationship(back_populates="documents")

    application_id: Optional[UUID] = Field(foreign_key="application.id", ondelete="SET NULL", index=False, nullable=True)
    application: "Application" = Relationship(back_populates="documents")


class Candidate(CandidatesOrganizationModel, table=True):
    marksmen_uid: Optional[str] = Field(index=True)

    first_name: str = Field(index=True)
    last_name: str = Field(index=True)
    email: str = Field(index=True)
    phone_number: Optional[str] = Field(index=True)
    city: Optional[str] = Field(index=True)
    country: Optional[str] = Field(index=True)
    documents: list["Document"] = Relationship(back_populates="candidate")
    applications: list["Application"] = Relationship(back_populates="candidate")


class Application(CandidatesOrganizationModel, table=True):
    vacancy_id: UUID = Field(index=True, nullable=True)
    vacancy_title: str = Field(index=True)
    vacancy_reference_number: str = Field(index=True)
    screening_answers: dict = Field(sa_column=Column(JSON))
    status: str = Field(index=True)

    source_id: Optional[UUID] = Field(foreign_key="source.id", ondelete="SET NULL", index=False, nullable=True)
    source: "Source" = Relationship(back_populates="applications")

    candidate_id: Optional[UUID] = Field(foreign_key="candidate.id", ondelete="SET NULL", index=True, nullable=True)
    candidate: Candidate = Relationship(back_populates="applications")

    documents: list["Document"] = Relationship(back_populates="application")


class Source(CandidatesOrganizationModel, table=True):
    source: str = Field(index=True)
    event_id: Optional[str] = Field(index=True)
    campaign_id: Optional[UUID] = Field(index=True)

    applications: list["Application"] = Relationship(back_populates="source")



from uuid import UUID
from datetime import datetime
from typing import Optional
from pydantic import BaseModel

from .models import Link as LinkModel, Integration as IntegrationModel, IntegrationType
from ..general.dataclasses import ResolvableModel, ResolvableCompanyModel, FileData
from talentro.services.clients import MSClient


# Integration object
class IntegrationInfo(ResolvableModel):
    id: UUID
    created_at: datetime
    updated_at: Optional[datetime]

    name: str
    icon: str
    type: IntegrationType
    tag: Optional[str]
    description: str

    @staticmethod
    async def resolve_object(object_id: UUID) -> "IntegrationInfo | None":
        result = await MSClient.integrations().get(f"integrations/{object_id}/resolve")

        if result.status_code != 200:
            return None

        return IntegrationInfo(**result.json())

    @classmethod
    async def from_model(cls: "IntegrationInfo", model: IntegrationModel) -> "IntegrationInfo":
        return cls(
            id=model.id,
            created_at=model.created_at,
            updated_at=model.updated_at,
            name=model.name,
            icon=model.icon,
            type=model.type,
            tag=model.tag,
            description=model.description,
        )


class IntegrationConfig(BaseModel):
    id: UUID
    created_at: datetime
    updated_at: Optional[datetime]

    name: str
    icon: str
    type: IntegrationType
    tag: str | None
    enabled: bool | None
    description: str | None
    code_reference: str
    setup_config: dict
    order: int

    @classmethod
    async def from_model(cls: "IntegrationConfig", model: IntegrationModel) -> "IntegrationConfig":
        return cls(**model.model_dump())


# Link object
class LinkInfo(ResolvableCompanyModel):
    id: UUID
    created_at: datetime
    updated_at: Optional[datetime]
    organization: UUID

    name: str
    status: str
    integration: IntegrationInfo | None

    @staticmethod
    async def resolve_object(object_id: UUID, organization_id: UUID) -> "LinkInfo | None":
        result = await MSClient.integrations().get(f"links/{object_id}/resolve", headers={"X-Organization-ID": str(organization_id)})

        if result.status_code != 200:
            return None

        return LinkInfo(**result.json())

    @classmethod
    async def from_model(cls: "LinkInfo", model: LinkModel) -> "LinkInfo | None":
        integration = await IntegrationInfo.resolve_object(model.integration_id)

        return cls(
            id=model.id,
            created_at=model.created_at,
            updated_at=model.updated_at,
            organization=model.organization,
            name=model.name,
            status=model.status,
            integration=integration,
        )


class LinkConfig(BaseModel):
    id: UUID
    created_at: datetime
    updated_at: Optional[datetime]
    organization: UUID

    name: str
    status: str
    auth_config: dict
    integration_id: UUID

    @classmethod
    async def from_model(cls: "LinkConfig", model: LinkModel) -> "LinkConfig":
        return cls(**model.model_dump())


class AdPublishResult(BaseModel):
    campaign_id: str
    success: bool
    code: str | None = None
    ad_id: str | None = None
    message: str | None = None


class PersonData(BaseModel):
    first_name: str
    last_name: str
    email: Optional[str] = None
    phone_number: Optional[str] = None
    country: Optional[str] = None
    city: Optional[str] = None


class SourceData(BaseModel):
    source: str
    applied_on: datetime
    event_id: Optional[str] = None
    campaign_id: Optional[str] = None


class ApplicationDetails(BaseModel):
    vacancy_id: Optional[str] = None
    vacancy_reference: Optional[str] = None
    vacancy_title: Optional[str] = None
    cv_file_data: Optional[FileData] = None
    motivation_file_data: Optional[FileData] = None
    answers: dict


class IncomingApplicationData(BaseModel):
    person: PersonData
    source: SourceData
    application: Optional[ApplicationDetails] = None



from datetime import datetime
from typing import Optional
from uuid import UUID

from pydantic import BaseModel

from talentro.acquisition.dataclasses import CampaignInfo
from talentro.vacancies.dataclasses import VacancyInfo
from talentro.candidates.models import Application as ApplicationModel, Document as DocumentModel, Candidate as CandidateModel


class CandidateInfo(BaseModel):
    id: UUID
    created_at: datetime
    updated_at: Optional[datetime]
    organization: UUID

    first_name: Optional[str] = None
    last_name: Optional[str] = None
    email: Optional[str] = None
    phone_number: Optional[str] = None
    city: Optional[str] = None
    country: Optional[str] = None

    @classmethod
    async def from_model(cls: "CandidateInfo", model: CandidateModel) -> 'CandidateInfo':
        return CandidateInfo(
            id=model.id,
            created_at=model.created_at,
            updated_at=model.updated_at,
            organization=model.organization,

            first_name=model.first_name,
            last_name=model.last_name,
            email=model.email,
            phone_number=model.phone_number,
            city=model.city,
            country=model.country,
        )



class ApplicationData(BaseModel):
    status: str
    source: str
    candidate_id: UUID
    vacancy_reference_number: str
    screening_answers: dict


class Application(ApplicationData):
    id: UUID
    created_at: datetime
    updated_at: Optional[datetime]
    organization: UUID


class DocumentInfo(BaseModel):
    id: UUID
    created_at: datetime
    updated_at: Optional[datetime]
    organization: UUID

    type: str
    blob_name: str

    @classmethod
    async def from_model(cls: "DocumentInfo", model: DocumentModel) -> 'DocumentInfo':
        return DocumentInfo(
            id=model.id,
            created_at=model.created_at,
            updated_at=model.updated_at,
            organization=model.organization,
            type=model.type,
            blob_name=model.blob_name
        )


class ApplicationInfo(BaseModel):
    id: UUID
    created_at: datetime
    updated_at: Optional[datetime]
    organization: UUID

    status: str
    screening_answers: dict

    vacancy: Optional[VacancyInfo]
    campaign: Optional[CampaignInfo]
    candidate: Optional[CandidateInfo]

    documents: list[DocumentInfo]

    @classmethod
    async def from_model(cls: "ApplicationInfo", model: ApplicationModel) -> 'ApplicationInfo':
        if model.source:
            campaign = await CampaignInfo.resolve_object(model.source.campaign_id, model.organization)
        else:
            campaign = None

        if model.vacancy_id:
            vacancy = await VacancyInfo.resolve_object(model.vacancy_id, model.organization)
        else:
            vacancy = None

        if model.candidate:
            candidate = await CandidateInfo.from_model(model.candidate)
        else:
            candidate = None

        return cls(
            id=model.id,
            created_at=model.created_at,
            updated_at=model.updated_at,
            organization=model.organization,

            screening_answers=model.screening_answers,
            status=model.status,

            vacancy=vacancy,
            campaign=campaign,
            candidate=candidate,

            documents=[await DocumentInfo.from_model(document) for document in model.documents],
        )
